<?php
/**
 * Cost of Goods for WooCommerce - Compatibility with WooCommerce Food.
 *
 * @see https://exthemes.net/woocommerce-food/
 *
 * @version 2.8.7
 * @since   2.8.2
 * @author  WPFactory
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Cost_of_Goods_Pro_Compatibility_WC_Food' ) ) {

	class Alg_WC_Cost_of_Goods_Pro_Compatibility_WC_Food {

		/**
		 * is_compatibility_option_enabled.
		 *
		 * @since 2.8.2
		 *
		 * @var null
		 */
		protected $is_fixed_option_costs_compatibility_enabled = null;

		/**
		 * cost_values.
		 *
		 * @since 2.8.2
		 *
		 * @var int
		 */
		protected $food_order_costs = array();

		/**
		 * Constructor.
		 *
		 * @version 2.8.2
		 * @since   2.8.2
		 */
		function __construct() {
			// Meta box.
			add_action( 'add_meta_boxes', array( $this, 'add_costs_meta_box_to_product_and_food_global_options' ) );
			add_action( 'save_post_product', array( $this, 'save_meta_box_option_cost' ), PHP_INT_MAX, 2 );
			add_action( 'save_post_exwo_glboptions', array( $this, 'save_meta_box_option_cost' ), PHP_INT_MAX, 2 );
			// Add meta to cart.
			add_filter( 'woocommerce_add_cart_item_data', array( $this, 'add_cart_item_data' ), 10, 2 );
			// Append COG fixed costs to "exwo_options" meta.
			add_filter( 'get_post_metadata', array( $this, 'append_fixed_costs_to_exwo_options' ), 10, 4 );
			// Add fixed costs to order item meta.
			add_action( 'woocommerce_new_order_item', array( $this, 'add_product_custom_field_to_order_item_meta' ), 9, 3 );
			// Adjust order costs and profit.
			add_filter( 'alg_wc_cog_update_order_item_values', array( $this, 'adjust_order_item_values' ), 10, 2 );
			add_action( 'alg_wc_cog_update_order_values_action', array( $this, 'save_cost_values' ) );
			// Order metabox.
			add_filter( 'alg_wc_cog_cost_meta_keys', array( $this, 'add_order_meta_box_meta_key' ) );
		}

		/**
		 * add_order_meta_box_meta_key.
		 *
		 * @version 2.8.2
		 * @since   2.8.2
		 *
		 * @param $keys
		 *
		 * @return mixed
		 */
		function add_order_meta_box_meta_key( $keys ) {
			if ( $this->is_fixed_option_costs_compatibility_enabled() ) {
				$keys['_alg_wc_cog_food_options_fixed_costs_total'] = __( 'Food costs', 'cost-of-goods-for-woocommerce' );
			}
			return $keys;
		}

		/**
		 * save_cost_values.
		 *
		 * @version 3.0.2
		 * @since   2.8.2
		 *
		 * @param $order_info
		 */
		function save_cost_values( $order_info ) {
			if ( ! empty( $this->food_order_costs ) ) {
				$item_costs_values = array_sum( $this->food_order_costs );
				$order             = $order_info['order'];
				$order_id          = $order_info['order_id'];
				$item_costs_values = (float) apply_filters( 'alg_wc_cog_food_options_fixed_costs_total', $item_costs_values, $order );
				$order->update_meta_data( '_alg_wc_cog_food_options_fixed_costs_total', (float) $item_costs_values );
				$order->save();
			}
		}

		/**
		 * adjust_order_item_values.
		 *
		 * @version 2.8.2
		 * @since   2.8.2
		 *
		 * @param $values
		 * @param $order_item_info
		 *
		 * @return mixed
		 * @throws Exception
		 */
		function adjust_order_item_values( $values, $order_item_info ) {
			if ( $this->is_fixed_option_costs_compatibility_enabled() ) {
				$item_id         = $order_item_info['item_id'];
				$item            = $order_item_info['item'];
				$costs_item_meta = wc_get_order_item_meta( $item_id, '_alg_wc_cog_food_options_fixed_costs', true );
				if ( ! empty( $costs_item_meta ) ) {
					$costs_total = 0;
					foreach ( $costs_item_meta as $post_id => $options ) {
						foreach ( $options as $option_id => $suboptions ) {
							$costs_total += array_sum( $suboptions );
						}
					}
					$costs_total *= $item->get_quantity();
					if ( ! isset( $this->food_order_costs[ $item_id ] ) ) {
						$this->food_order_costs[ $item_id ] = $costs_total;
					} else {
						$this->food_order_costs[ $item_id ] += $costs_total;
					}
					$values['profit']     -= $costs_total;
					$values['total_cost'] += $costs_total;
				}
			}
			return $values;
		}

		/**
		 * add_product_custom_field_to_order_item_meta.
		 *
		 * @version 2.8.7
		 * @since   2.8.2
		 *
		 * @param $item_id
		 * @param $item
		 * @param $order_id
		 *
		 * @throws Exception
		 */
		function add_product_custom_field_to_order_item_meta( $item_id, $item, $order_id ) {
			if (
				$this->is_fixed_option_costs_compatibility_enabled() &&
				property_exists( $item, 'legacy_values' ) &&
				! empty( $fixed_costs = $item->legacy_values['_alg_wc_cog_options_fixed_costs'] )
			) {
				$result = wc_update_order_item_meta( $item_id, '_alg_wc_cog_food_options_fixed_costs', $fixed_costs );
			}
		}

		/**
		 * get_post_metadata.
		 *
		 * @version 2.8.3
		 * @since   2.8.2
		 *
		 * @param $metadata
		 * @param $object_id
		 * @param $meta_key
		 * @param $single
		 *
		 * @return array
		 */
		function append_fixed_costs_to_exwo_options( $metadata, $object_id, $meta_key, $single ) {
			$meta_needed = 'exwo_options';
			if (
				$this->is_fixed_option_costs_compatibility_enabled() &&
				isset( $meta_key ) &&
				$meta_needed === $meta_key
			) {
				remove_filter( 'get_post_metadata', array( $this, 'append_fixed_costs_to_exwo_options' ), 10 );
				$original_meta = get_post_meta( $object_id, $meta_needed, $single );
				if ( ! empty( $original_meta ) && is_array( $original_meta ) ) {
					add_filter( 'get_post_metadata', array( $this, 'append_fixed_costs_to_exwo_options' ), 10, 4 );
					$fixed_costs = get_post_meta( $object_id, '_alg_wc_cog_options_fixed_costs', true );
					foreach ( $original_meta as $k => $v ) {
						$original_meta[ $k ]['_alg_wc_cog_options_fixed_costs'] = $fixed_costs[ $k ];
						$original_meta[ $k ]['_alg_wc_cog_option_post_id']      = $object_id;
						$original_meta[ $k ]['_alg_wc_cog_option_index']        = $k;
					}
					$final_meta = array( $original_meta );
					return $final_meta;
				} else {
					return null;
				}
			}
			return $metadata;
		}

		/**
		 * add_cart_item_data.
		 *
		 * @version 2.8.2
		 * @since   2.8.2
		 *
		 * @param $cart_item_data
		 * @param $product_id
		 *
		 * @return mixed
		 */
		function add_cart_item_data( $cart_item_data, $product_id ) {
			if ( $this->is_fixed_option_costs_compatibility_enabled() && class_exists( 'EX_Woo_Custom_Option' ) ) {
				$data_options  = exwoo_get_options( $product_id );
				$cog_item_data = array();
				foreach ( $data_options as $key => $options ) {
					$data_exts = isset( $_POST[ 'ex_options_' . $key ] ) ? $_POST[ 'ex_options_' . $key ] : '';
					if ( ! empty( $data_exts ) ) {
						$cog_item_data[ $data_options[ $key ]['_alg_wc_cog_option_post_id'] ][ $data_options[ $key ]['_alg_wc_cog_option_index'] ] = array_intersect_key( $data_options[ $key ]['_alg_wc_cog_options_fixed_costs'], array_flip( $data_exts ) );
					}
				}
				if ( ! empty( $cog_item_data ) ) {
					$cart_item_data['_alg_wc_cog_options_fixed_costs'] = $cog_item_data;
				}
			}
			return $cart_item_data;
		}

		/**
		 * save_option_cost.
		 *
		 * @version 2.8.3
		 * @since   2.8.2
		 *
		 * @param $post_id
		 */
		function save_meta_box_option_cost( $post_id ) {
			if (
				$this->is_fixed_option_costs_compatibility_enabled() &&
				isset( $_POST['alg_wc_cog_option_fixed_cost'] ) &&
				! empty( $options_fixed_costs = $_POST['alg_wc_cog_option_fixed_cost'] )
			) {
				if ( isset( $_POST['exwo_options'] ) ) {
					foreach ( $_POST['exwo_options'] as $option_key => $option ) {
						if ( isset( $option['_options'] ) && ! empty( $option['_options'] ) ) {
							$suboption_order                    = array_filter( $option['_options'], function ( $item ) {
								return array_filter( $item );
							} );
							$options_fixed_costs[ $option_key ] = array_replace( array_flip( array_keys( $suboption_order ) ), $options_fixed_costs[ $option_key ] );
						}
					}
					update_post_meta( $post_id, '_alg_wc_cog_options_fixed_costs', $options_fixed_costs );
				}
			}
		}

		/**
		 * add_costs_meta_box_to_product_and_food_global_options
		 *
		 * @version 2.8.2
		 * @since   2.8.2
		 */
		function add_costs_meta_box_to_product_and_food_global_options() {
			if ( ! apply_filters( 'alg_wc_cog_create_product_meta_box_validation', true ) ) {
				return;
			}
			if ( $this->is_fixed_option_costs_compatibility_enabled() ) {
				add_meta_box( 'alg-wc-cog-food-compatibility-cmb',
					__( 'Option(s) costs', 'cost-of-goods-for-woocommerce' ),
					array( $this, 'display_option_costs_table' ),
					array( 'product', 'exwo_glboptions' ),
					'side'
				);
			}
		}

		/**
		 * display_option_costs_table.
		 *
		 * @version 2.8.2
		 * @since   2.8.2
		 *
		 * @param $post
		 */
		function display_option_costs_table( $post ) {
			if ( empty( $post ) ) {
				return;
			}
			$exwo_options = get_post_meta( $post->ID, 'exwo_options', true );
			if ( ! empty( $exwo_options ) ) {
				$fixed_costs = get_post_meta( $post->ID, '_alg_wc_cog_options_fixed_costs', true );
				echo '<p class="description">Fixed costs</p>';
				$table = '<table class="widefat striped"><tbody>';

				foreach ( $exwo_options as $option_key => $option ) {
					if ( isset( $option['_options'] ) && ! empty( $option['_options'] ) ) {
						$i = 0;
						foreach ( $option['_options'] as $sub_option_key => $sub_option ) {
							$fixed_costs_sub_option_key = isset( $fixed_costs[ $option_key ] ) ? key( $fixed_costs[ $option_key ] ) : 0;
							$input_value                = isset( $fixed_costs[ $option_key ][ $fixed_costs_sub_option_key ] ) && ! empty( $fixed_costs[ $option_key ][ $fixed_costs_sub_option_key ] ) ? $fixed_costs[ $option_key ][ $fixed_costs_sub_option_key ] : 0;
							$table                      .= '<tr>';
							$table                      .= '<th><label for="alg_wc_cog_option_fixed_cost_' . $option_key . '_' . $i . '">' . $sub_option['name'] . '</label>' . '</th>';
							$table                      .= '<td style="text-align:left"><input style="max-width:150px" name="alg_wc_cog_option_fixed_cost[' . $option_key . '][' . $i . ']" id="alg_wc_cog_option_fixed_cost_' . $option_key . '_' . $i . '" class="short wc_input_price" value="' . esc_attr( $input_value ) . '" type="number" step="0.0001" min="0"></td>';
							$table                      .= '</tr>';
							$i ++;
							if ( isset( $fixed_costs[ $option_key ] ) ) {
								next( $fixed_costs[ $option_key ] );
							}
						}
					}
				}
				$table .= '</tbody></table>';
				echo $table;
			}
		}

		/**
		 * is_add_stock_enabled.
		 *
		 * @version 2.8.2
		 * @since   2.8.2
		 *
		 * @return bool|null
		 */
		function is_fixed_option_costs_compatibility_enabled() {
			if ( is_null( $this->is_fixed_option_costs_compatibility_enabled ) ) {
				$this->is_fixed_option_costs_compatibility_enabled = ( 'yes' === get_option( 'alg_wc_cog_comp_wc_food_fixed_options_costs', 'no' ) );
			}
			return $this->is_fixed_option_costs_compatibility_enabled;
		}
	}
}

return new Alg_WC_Cost_of_Goods_Pro_Compatibility_WC_Food();